#!/usr/bin/env python

'''

    @version $Id: aamaprotator,v 1.5 2005/10/19 05:52:58 davidfancella Exp $

    This file is part of aamaprotator.

    aamaprotator is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    aamaprotator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with aamaprotator; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    aamaprotator is copyright 2005 by David Fancella

'''

from ConfigParser import ConfigParser
from libaamaprotator import paths
import sys, os
from time import strftime
from libaamaprotator import server_settings

# only used for generated maps, will be appended to ensure unique names for maps
# that are generated
mapCounter = 0

class aamConfig(object):
    def __init__(self, path):
        self.config = ConfigParser()

        #try:
        self.config.read(path)
            
        if self.config.has_section('main'):
            if not self.config.has_option('main','match_end'):
                self.config.set('main','match_end','\\[0\\] Overall Winner*')

            if self.config.has_option('main','armaprefix'):
                paths.SetArmaPrefix(self.config.get('main','armaprefix'))
            if self.config.has_option('main', 'armaconfig'):
                paths.SetArmaSettingsDir(self.config.get('main','armasettings'))

            # Just a measure to make sure the config object always has uptodate
            # information, probably not needed.
            self.config.set('main', 'armasettings', paths.GetArmaSettingsDir() )
            self.config.set('main', 'armaprefix', paths.GetArmaPrefix() )

            if not self.config.has_option('main','template'):
                self.config.set('main', 'template', os.path.join(paths.GetConfigDir(), 'everytime.tpl') )

            # These all have to be set after armaprefix is set in the paths module
            if not self.config.has_option('main','everytime'):
                self.config.set('main', 'everytime', os.path.join(paths.GetArmaPrefix(), 'etc', 'games', 'armagetronad-dedicated', 'everytime.cfg') )

            if not self.config.has_option('main','armalog'):
                self.config.set('main', 'armalog', os.path.join(paths.GetArmaPrefix(), 'var','games','armagetronad-dedicated','log','armagetronad.log') )
        else:
            paths.LogLine("Config file is broken, it doesn't include a main section.")
            sys.exit()
        #except:
        #    paths.LogLine("Config file is broken, it couldn't be loaded.")
            
        # setup the list of maps
        sectionlist = self.config.sections()
        self.maplist = []
        
        for section in sectionlist:
            if section != 'main':
                self.maplist.append(section)

    # Gets an option from the main section
    def get(self, option, default=None):
        if self.config.has_option('main', option):
            return self.config.get('main', option)
        else:
            return default

    def getFloat(self, option, default=1.0):
        if self.config.has_option('main', option):
            return self.config.getfloat('main',option)
        else:
            return default
        
    # Returns a map.  If the map doesn't exist, returns the default map for the game
    def getMap(self, theMap):
        if self.config.has_section(theMap):
            return aamMap(self.config.items(theMap))
        else:
            return aamMap()

    def getMapList(self):
        return self.maplist

class aamMap(object):
    def __init__(self, theMap = None):
        self.mapinfo = { 'type': 'flat' }
        
        if theMap is not None:
            for item in theMap:
                self.mapinfo[item[0]] = item[1]
        else:
            self.mapinfo['author'] = 'Anonymous'
            self.mapinfo['path'] = 'Anonymous/polygon/regular/square-1.0.1.aamap.xml'

    def Generate(self):
        global mapCounter
        mapCounter += 1
        theFilename = strftime("%a%b%m%d%H%M%S") + str(mapCounter) + ".aamap.xml"
        thePath = os.path.join(paths.GetArmaPrefix(), 'share', 'games', 'armagetronad-dedicated', 'resource', self.mapinfo['resource'])
        theFullPath = os.path.join(thePath, theFilename)
        theReturnPath = os.path.join(self.mapinfo['resource'], theFilename)
        theCommand = self.mapinfo['command']
        theCommand = theCommand.replace('%filename%',theFullPath)
        os.system(theCommand)
        return theReturnPath

    def GetType(self):
        return self.mapinfo['type']

    def GetPath(self):
        return self.mapinfo['path']

    def GetSettings(self):
        settingsObject = server_settings.aaMapSettings()
        
        if self.mapinfo.has_key('size'):
            settingsObject.Set('SIZE_FACTOR', self.mapinfo['size'])
            settingsObject.Set('SP_SIZE_FACTOR', self.mapinfo['size'])

        if self.mapinfo.has_key('settings'):
            settingsfilelist = self.mapinfo['settings'].split(",")
            for a in range(len(settingsfilelist)):
                settingsfilelist[a] = settingsfilelist[a].strip()
            settingsObject.LoadFiles(settingsfilelist)
                
        return settingsObject
        
    def GetCredits(self):
        mapAuthor = None
        mapCommissioner = None
        theOutput = ''
        if self.mapinfo.has_key('author'):
            mapAuthor = self.mapinfo['author']
        if self.mapinfo.has_key('commissioner'):
            mapCommissioner = self.mapinfo['commissioner']
        if mapAuthor is not None:
            theOutput = 'This map written by ' + mapAuthor
            if mapCommissioner is not None:
                theOutput += ', commissioned by ' + mapCommissioner
            theOutput += '.'
        return theOutput

        