#!/usr/bin/env python

'''

    @version $Id: aamaprotator,v 1.5 2005/10/19 05:52:58 davidfancella Exp $

    This file is part of aamaprotator.

    aamaprotator is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    aamaprotator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with aamaprotator; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    aamaprotator is copyright 2005 by David Fancella

'''

import random, time, os, re, sys
from config import aamConfig
from libaamaprotator import paths
from libaamaprotator import filetail
from libaamaprotator import server_settings

def main():
    paths.LogLine("Program started.")
    config = aamConfig(paths.GetConfigFile())
    
    currentmap = 0
    # setup the regexes
    matchEndRE = re.compile(config.get('match_end') )

    # read the everytime.tpl file
    everytime = []
    file = paths.OpenFile(config.get('template'), 'r')

    if file is not None:
        for line in file:
            everytime.append(line)
    
        file.close()
    else:
        paths.LogLine("Could not open template file, exiting.")
        sys.exit()

    # read armagetronad-dedicated's settings files to determine the defaults to be
    # used.  Failure to read is not a serious problem--yet.
    defaultsettings = []
    aaSettings = server_settings.ArmaSettings(paths.GetArmaSettingsDir())
    defaultsettings = aaSettings.settings
    
    maplist = config.getMapList()
    random.shuffle(maplist)

    min_sleep = config.getFloat('min_sleep', 1.0)
    max_sleep = config.getFloat('max_sleep', 60.0)
    sleep_interval = config.getFloat('sleep_interval', 1.0)

    logFile = filetail.Tail(config.get('armalog'), True,
                 min_sleep,
                 sleep_interval,
                 max_sleep)
    
    for inline in logFile:
        result = matchEndRE.match(inline)
        
        if(result is not None):
            if currentmap == len(maplist):
                currentmap = 0
                random.shuffle(maplist)
            currentmapObj = config.getMap(maplist[currentmap])
            
            paths.LogLine("Rotating map.  New Map: " + maplist[currentmap])
            
            outfile = paths.OpenFile(config.get('everytime'), 'w')

            if outfile is not None:
                for line in everytime:
                    # first strip comments
                    codeLine = line[0:line.find("#")]
                    codeLine = codeLine.strip()
                    if codeLine == 'AAMAPROTATOR':
                        # restore to defaults all settings changed by the last map
                        # On the first run, it will use all settings in the config files
                        # which is a-ok.
                        for ds in defaultsettings:
                            theOutput = ds[0] + " " + ds[1] + "\n"
                            outfile.write(theOutput)
                        if currentmapObj.GetType() == 'flat':
                            theOutput = 'MAP_FILE ' + currentmapObj.GetPath() + "\n"
                            outfile.write(theOutput)
                        elif currentmapObj.GetType() == 'script':
                            theOutput = 'MAP_FILE ' + currentmapObj.Generate() + "\n"
                            outfile.write(theOutput)
    
                        theNewSettings = currentmapObj.GetSettings()
                        for setting in theNewSettings.GetSettings():
                            theOutput = setting[0] + " " + setting[1] + "\n"
                            outfile.write(theOutput)

                        # now set the default settings to only the settings that were
                        # changed by this map.  This makes it so that a server admin
                        # can still change settings with ingame admin that are not
                        # changed by any settings file
                        # todo: make it so that the script rewrites itself after
                        # everytime.cfg has been loaded
                        #
                        # doesn't work for some reason, so it's disabled
                        #   so I could put it up on my server
                        #
                        #defaultsettings = []
                        #defaultsettings = aaSettings.GetDefaultDiff(theNewSettings)
                        
                        theCredits = currentmapObj.GetCredits()
                        if theCredits != '':
                            theOutput = 'CONSOLE_MESSAGE ' + currentmapObj.GetCredits()
                            outfile.write(theOutput)
                    else:
                        outfile.write(line)
                # finished writing the file
                outfile.close()
            currentmap += 1
        else:
            pass
        result = None
        
