#!/usr/bin/env python

'''

    @version $Id: aamaprotator,v 1.5 2005/10/19 05:52:58 davidfancella Exp $

    This file is part of aamaprotator.

    aamaprotator is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    aamaprotator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with aamaprotator; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    aamaprotator is copyright 2005 by David Fancella

'''

# this is just a series of functions to keep the main program from having to deal with
# where everything is

import os
from time import asctime

# These get set in Initialize and are only initialized here to show you what they do
paths = { 'etc' : '/etc',
          'var' : '/usr/var',
          'share' : '/usr/share',
          'armaprefix' : '/usr/local',
          'armasettings' : '/usr/local/etc/games/armagetronad-dedicated' }

def Initialize(prefix):
    global paths

    head, tail = os.path.split(prefix)
    if tail == 'bin':
        if os.path.exists(os.path.join(head, 'etc', 'games', 'aamaprotator')):
            paths['etc'] = os.path.join(head, 'etc', 'games', 'aamaprotator')
        elif os.path.exists(os.path.join(head, 'etc', 'aamaprotator')):
            paths['etc'] = os.path.join(head, 'etc', 'aamaprotator')
        elif os.path.exists(os.path.join('/etc', 'games', 'aamaprotator')):
            paths['etc'] = os.path.join('/etc', 'games', 'aamaprotator')
        elif os.path.exists(os.path.join('/etc', 'aamaprotator')):
            paths['etc'] = os.path.join('/etc', 'aamaprotator')

        if os.path.exists(os.path.join(head, 'var', 'games', 'aamaprotator')):
            paths['var'] = os.path.join(head, 'var', 'games', 'aamaprotator')
        elif os.path.exists(os.path.join(head, 'var', 'aamaprotator')):
            paths['var'] = os.path.join(head, 'var', 'aamaprotator')
        elif os.path.exists(os.path.join(head, 'var')):
            paths['var'] = os.path.join(head, 'var')
        elif os.path.exists(os.path.join('/var', 'games', 'aamaprotator')):
            paths['var'] = os.path.join('/var', 'games', 'aamaprotator')
        elif os.path.exists(os.path.join('/var', 'aamaprotator')):
            paths['var'] = os.path.join('/var', 'aamaprotator')
        elif os.path.exists(os.path.join('/var')):
            paths['var'] = os.path.join('/var')
    else:
        head = prefix
        if os.path.exists(os.path.join(head, 'config')):
            paths['etc'] = os.path.join(head, 'config')
        elif os.path.exists(os.path.join(head, 'etc', 'games', 'aamaprotator')):
            paths['etc'] = os.path.join(head, 'etc', 'games', 'aamaprotator')
        elif os.path.exists(os.path.join(head, 'etc', 'aamaprotator')):
            paths['etc'] = os.path.join(head, 'etc', 'aamaprotator')
        elif os.path.exists(os.path.join('/etc', 'games', 'aamaprotator')):
            paths['etc'] = os.path.join('/etc', 'games', 'aamaprotator')
        elif os.path.exists(os.path.join('/etc', 'aamaprotator')):
            paths['etc'] = os.path.join('/etc', 'aamaprotator')
    
        if os.path.exists(os.path.join(head, 'var', 'games', 'aamaprotator')):
            paths['var'] = os.path.join(head, 'var', 'games', 'aamaprotator')
        elif os.path.exists(os.path.join(head, 'var', 'aamaprotator')):
            paths['var'] = os.path.join(head, 'var', 'aamaprotator')
        elif os.path.exists(os.path.join('/var', 'games', 'aamaprotator')):
            paths['var'] = os.path.join('/var', 'games', 'aamaprotator')
        elif os.path.exists(os.path.join('/var', 'aamaprotator')):
            paths['var'] = os.path.join('/var', 'aamaprotator')
        elif os.path.exists(os.path.join(head)):
            paths['var'] = os.path.join(head)
    
    # First check if arma is in the same prefix we're in, then check 2 standard locations
    # If this doesn't identify anything, arma's prefix should be set in the config file
    # The rest of aamaprotator will use this module to get its paths, so make sure the
    # config loader sets the paths appropriately that are defined in the config file!
    if os.path.exists(os.path.join(head, 'bin', 'armagetronad-dedicated')):
        paths['armaprefix'] = head
    elif os.path.exists(os.path.join('/usr', 'bin', 'armagetronad-dedicated')):
        paths['armaprefix'] = os.path.join('/usr')
    elif os.path.exists(os.path.join('/usr', 'local', 'bin', 'armagetronad-dedicated')):
        paths['armaprefix'] = os.path.join('/usr', 'local')
    
    # Now do the same thing for the settings directory, looking specifically for
    # settings_dedicated.cfg
    if os.path.exists(os.path.join(head, 'etc', 'games', 'armagetronad-dedicated')):
        paths['armasettings'] = os.path.join(head, 'etc', 'games', 'armagetronad-dedicated')
    elif os.path.exists(os.path.join('/usr', 'etc', 'games', 'armagetronad-dedicated')):
        paths['armasettings'] = os.path.join('/usr', 'etc', 'games', 'armagetronad-dedicated')
    elif os.path.exists(os.path.join('/usr', 'local', 'etc', 'games', 'armagetronad-dedicated')):
        paths['armasettings'] = os.path.join('/usr', 'local', 'etc', 'games', 'armagetronad-dedicated')

def OpenFile(path, mode):
    file = None
    try:
        file = open(path, mode)
    except:
        LogLine("Couldn't open file '" + str(path) + "' in mode '" + mode + "'.  Sorry.")

    return file

def GetLogFile():
    global paths
    return os.path.join(paths['var'], 'log', 'aamaprotator.log')

def LogLine(someline):
    theline = asctime() + ": " + someline + "\n"
    try:
        file = open(GetLogFile(), 'a')
        file.write(theline)
        file.close()
    except IOError:
        print theline

def GetConfigDir():
    global paths
    return paths['etc']

def GetSettingsDir():
    return os.path.join(GetConfigDir(), 'settings')

def GetConfigFile():
    return os.path.join(GetConfigDir(), 'aamaprotator.conf')

def GetArmaPrefix():
    global paths
    return paths['armaprefix']

def GetArmaSettingsDir():
    global paths
    return paths['armasettings']

def SetArmaPrefix(path):
    global paths
    paths['armaprefix'] = path

def SetArmaSettingsDir(path):
    global paths
    paths['armasettings'] = path
