#!/usr/bin/python

#!/usr/bin/env python

'''

    @version $Id: mazer.py,v 1.2 2005/11/02 08:16:02 dave Exp $

    This file is part of armagetron advanced maze generator.

    armagetron advanced maze generator is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    armagetron advanced maze generator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with armagetron advanced maze generator; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    armagetron advanced maze generator is copyright 2005 by David Fancella

'''

import random, sys, math

UP = 1
DOWN = 0
LEFT  = 2
RIGHT  = 3
studyingmaze = False

class Zone:
    x = None
    y = None
    radius = None
    growth = 0
    effect = 'win'

    def __init__(self, **args):
        self.x = args['x']
        self.y = args['y']
        self.radius = args['radius']
        if args.has_key('effect'):
            self.effect = args['effect']
        if args.has_key('growth'):
            self.growth = args['growth']
    
    def getAsXML(self):
        result = ""
                
        result += '<Zone effect="' + self.effect + '">' + "\n"
        result += '  <ShapeCircle radius="' + str(self.radius) + '" growth="' + str(self.growth) + '">' + "\n"
        result += '    <Point x="' + str(self.x) + '" y="' + str(self.y) + '" />' + "\n"
        result += "  </ShapeCircle>\n"
        result += "</Zone>\n"

        return result

class Spawn:
    x = None
    y = None
    direction = None

    def __init__(self, **args):
        self.x = args['x']
        self.y = args['y']
        self.direction = args['direction']
    
    def getAsXML(self):
        result = ""

        result += "<Spawn  x=\"" + str(self.x) + "\" y=\""+ str(self.y) + "\"  angle=\"" + str(self.direction) + "\" />\n"

        return result        

class Wall:
    x1 = None
    y1 = None
    x2 = None
    y2 = None
    height = None

    def __init__(self, **args):
        self.x1 = args['x1']
        self.y1 = args['y1']
        self.x2 = args['x2']
        self.y2 = args['y2']

        if args.has_key('height'):
            self.height = args['height']
        else:
            self.height = 2.0

    def __str__(self):
        return "(" + str(self.x1) + "," + str(self.y1) + ") : (" + str(self.x2) + "," + str(self.y2) + ")"
                    
    def getAsXML(self):
        result = ""

        result += "<Wall height=\"" + str(self.height) + "\">\n"
        result += '<Point x="' + str(self.x1) + '" y="' + str(self.y1) + '" />'
        result += "\n"
        result += '<Point x="' + str(self.x2) + '" y="' + str(self.y2) + '" />'
        result += "\n"
        result += "</Wall>\n"

        return result        

    def isHorizontal(self):
        return (self.y1 == self.y2)

    def isVertical(self):
        return (self.x1 == self.x2)
        
    # The comparison functions (except __com__) are meaningless for wall segments
    # not on the same parallel, so only compare walls that are on the same
    # parallel
    def __ge__(self, other):
        # First check for equality
        if self.__cmp__(other) == 0:
            return True
        else:
            return self.__gt__(other)

    # This is a basic implementation, it'll break easily under more complex
    # conditions outside this maze generator.
    def __gt__(self, other):
        if self.isHorizontal() and other.isHorizontal():
            if self.x1 > other.x1:
                return True
            return False                
        elif self.isVertical() and other.isVertical():
            if self.y1 > other.y1:
                return True
            return False
        else:
            return False

    def __le__(self, other):
        # First check for equality
        if self.__cmp__(other) == 0:
            return True
        else:
            return self.__lt__(other)

    def __lt__(self, other):
        if self.isHorizontal() and other.isHorizontal():
            if self.x1 < other.x1:
                return True
            return False                
        elif self.isVertical() and other.isVertical():
            if self.y1 < other.y1:
                return True
            return False
        else:
            return False
        
    # returns True if they are the same    
    def __cmp__(self, other):
        if self.x1 == other.x1:
            if self.y1 == other.y1:
                if self.x2 == other.x2:
                    if self.y2 == other.y2:
                        return 0
        # Maybe the walls are backwards, need to check that too
        if self.x1 == other.x2:
            if self.y1 == other.y2:
                if self.x2 == other.x1:
                    if self.y2 == other.y1:
                        return 0
        return 1

class Cell:
    spawn = False
    spawnDirection = None

    zone = None
    visited = False
    rightwall = True
    bottomwall = True
    topwall = True
    leftwall = True

    # These will be references to this cell's neighbors.  None means it has
    # no neighbors.
    topCell = None
    bottomCell = None
    rightCell = None
    leftCell = None

    def __init__(self):
        pass

    def GetSpawn(self, x, y, xlength, ylength):
        if self.spawn:
            spawnx = x * xlength + (float(xlength)/2.0)
            spawny = y * ylength + (float(ylength)/2.0)
            if self.spawnDirection == UP:
                spawndir = 90
            elif self.spawnDirection == DOWN:
                spawndir = -90
            elif self.spawnDirection == RIGHT:
                spawndir = 0
            elif self.spawnDirection == LEFT:
                spawndir = 180
            else:
                raise RuntimeError, "Spawn point needs a direction"
            return Spawn(x=spawnx, y=spawny, direction=spawndir)
        else:
            return None

    def GetZone(self, x, y, xlength, ylength):
        if self.zone is not None:
            zonex = float(x) * float(xlength) + (float(xlength)/2.0)
            zoney = float(y) * float(ylength) + (float(ylength)/2.0)
            return Zone(x=zonex, y=zoney,
                        radius = self.zone[0],
                        effect=self.zone[1],
                        growth=self.zone[2] )
        return None

    def SetZone(self, radius, effect, growth=0.0):
        self.zone = (radius, effect, growth)
            
    def GetWalls(self, x, y, xlength, ylength, height=2.0):
        results = []
        if self.bottomwall:
            x1 = x * xlength
            y1 = y * ylength
            x2 = x1 + xlength
            y2 = y1
            results.append( Wall(
                            x1=x1, y1=y1,
                            x2=x2, y2=y2,
                            height = height ) )
        if self.topwall:
            x1 = x * xlength
            y1 = (y+1) * ylength
            x2 = x1 + xlength
            y2 = y1
            results.append( Wall(
                            x1=x1, y1=y1,
                            x2=x2, y2=y2,
                            height = height ) )
        if self.leftwall:
            x1 = x * xlength
            y1 = y * ylength
            x2 = x1
            y2 = y1 + ylength
            results.append( Wall(
                            x1=x1, y1=y1,
                            x2=x2, y2=y2,
                            height = height ) )
        if self.rightwall:
            x1 = (x+1) * xlength
            y1 = y * ylength
            x2 = x1
            y2 = y1 + ylength
            results.append( Wall(
                            x1=x1, y1=y1,
                            x2=x2, y2=y2,
                            height = height ) )
        return results
        
    # During maze generation, mark this cell visited
    def Visit(self):
        self.visited = True

    def inaccessible(self):
        if self.rightwall + self.topwall + self.bottomwall + self.leftwall == 4:
            return True
        return False

    def hasPaths(self, paths):
        global studyingmaze
        if (self.rightwall + self.topwall + self.bottomwall + self.leftwall) == paths:
            if not studyingmaze: print self.rightwall + self.topwall + self.bottomwall + self.leftwall, paths
            return True
        return False
        
    # Returns True if there is no wall in the indicated direction
    def canMove(self, direction):
        clear = False
        if direction == UP:
            if self.topwall is False:
                if self.topCell.bottomwall is False:
                    clear = True
        if direction == DOWN:
            if self.bottomwall is False:
                if self.bottomCell.topwall is False:
                    clear = True
        if direction == LEFT:
            if self.leftwall is False:
                if self.leftCell.rightwall is False:
                    clear = True
        if direction == RIGHT:
            if self.rightwall is False:
                if self.rightCell.leftwall is False:
                    clear = True
        return clear

    def setSpawn(self, direction):
        self.ripWall(direction)
        self.spawn = True
        self.spawnDirection = direction
    
    # Gets the cell that lies in the indicated direction
    def getCell(self, direction):
        if direction == UP:
            return self.topCell
        if direction == DOWN:
            return self.bottomCell
        if direction == LEFT:
            return self.leftCell
        if direction == RIGHT:
            return self.rightCell
        
    # During maze generation, get cells around that are unvisited
    def GetUnvisited(self):
        directions = []
        if self.topCell is not None and self.topCell.visited is False:
            directions.append(UP)
        if self.bottomCell is not None and self.bottomCell.visited is False:
            directions.append(DOWN)
        if self.leftCell is not None and self.leftCell.visited is False:
            directions.append(LEFT)
        if self.rightCell is not None and self.rightCell.visited is False:
            directions.append(RIGHT)
        return directions    
        
    def ripWall(self, direction):
        if direction == UP:
            if self.topCell is not None:
                self.topwall = False
                self.topCell.bottomwall = False
        if direction == DOWN:
            if self.bottomCell is not None:
                self.bottomwall = False
                self.bottomCell.topwall = False
        if direction == LEFT:
            if self.leftCell is not None:
                self.leftwall = False
                self.leftCell.rightwall = False
        if direction == RIGHT:
            if self.rightCell is not None:
                self.rightwall = False
                self.rightCell.leftwall = False

    def getBlocked(self):
        directions = []

        if self.topwall is True:
            directions.append(UP)
        if self.bottomwall is True:
            directions.append(DOWN)
        if self.leftwall is True:
            directions.append(LEFT)
        if self.rightwall is True:
            directions.append(RIGHT)
        return directions

class Maze:
    xUnitSize = 10
    yUnitSize = 10
    numRows = 25
    numCols = 25
    deadEnds = None
    # Fraction of dead ends to cull
    deadEndCull = 0.9
    # Fraction of one pathed cells to turn into two-path cells
    onePathCull = 0.12
    twoPathCull = 0.12
    spawnPoints = 8
    spawnRadius = 0.9
    wallHeight = 0.9
    game_mode = 1

    zones = None
    # The diameter used for placing zones if no positions are given
    # It's a ratio, actual diameter will be computed from it
    zoneDiameter = 0.8
    
    def __init__( self, **args ):
        if args.has_key("rows"):
            self.numRows = args['rows']
        if args.has_key("cols"):
            self.numCols = args['cols']
        if args.has_key("dead_end_cull"):
            self.deadEndCull = float(args['dead_end_cull'])
        if args.has_key('one_path_cull'):
            self.onePathCull = float(args['one_path_cull'])
        if args.has_key('two_path_cull'):
            self.twoPathCull = float(args['two_path_cull'])
        if args.has_key('wall_height'):
            self.wallHeight = float(args['wall_height'])
        if args.has_key('game_mode'):
            self.game_mode = int(args['game_mode'])
      
      # Pass it either a list of zones, or a single zone, description given:
        '''
            zone['loc'] = (x,y) (ratios, with 0.0 being left, 1.0 being right, etc, not arma coordinates)
            zone['effect'] = win, fortress, death
            zone['radius'] = radius
            zone['growth'] = growth
        '''
        if args.has_key("zone"):
            self.zones = args['zone']
        # spawn stuff currently not used
        if args.has_key("spawn_points"):
            self.spawnPoints = args['spawn_points']
        if args.has_key("spawn_radius"):
            self.spawnRadius = args['spawn_radius']
        
        self.xUnitSize = 500 / self.numRows
        self.yUnitSize = 500 / self.numCols
        
        self.deadEnds = []

        self.InitializeMaze()
    
    def InitializeMaze(self):
        self.maze = [None]*self.numRows
    
        for i in range(self.numRows):
            self.maze[i] = [None]*self.numCols
            for j in range(self.numCols):
                self.maze[i][j] = Cell()

        # For big mazes, the recursion can be really deep, so set the limit
        # to compensate for it
        # NOTE - You might run out of memory for mazes that are too big
        if self.numRows*self.numCols > sys.getrecursionlimit():
            sys.setrecursionlimit( self.numRows*self.numCols+5 )

        # Every cell needs to know about it's neighbors, that's fundamental
        # So iterate through the maze and tell each cell who it's neighbors are
        # This can't be done until the maze has been initially created
        for i in range(self.numRows):
            for j in range(self.numCols):
                if i > 0:
                    # The cell to the left
                    self.maze[i][j].leftCell = self.maze[i-1][j]
                if i < self.numRows-1:
                    # The cell to the right
                    self.maze[i][j].rightCell = self.maze[i+1][j]
                if j > 0:
                    # The cell to the top
                    self.maze[i][j].topCell = self.maze[i][j-1]
                if j < self.numCols-1:
                    # The cell to the bottom
                    self.maze[i][j].bottomCell = self.maze[i][j+1]
        

    def GenerateMaze(self):
        self.PlaceWalls()
        self.PlaceSpawnPoints()
        self.PlaceZones()
        self.finishedStatistics = self.StudyMaze()

    def PlaceWalls(self):
        # Choose a random starting point
        currCol = random.randrange(self.numCols)
        currRow = random.randrange(self.numRows)
        
        self._make_path( self.maze[currRow][currCol] )
        self.rawStatistics = self.StudyMaze()
        #self._cull(0, self.deadEndCull)
        self._cull(1, self.onePathCull)
        #self._cull(2, self.twoPathCull)
        self.unvisited = self._count_unvisited()
        self.inaccessible = self._count_inaccessible()
    
    def StudyMaze(self):
        stats = {
                 'dead_ends':0,
                 'two_open':0,
                 'three_open':0,
                 'four_open':0
                }
        global studyingmaze
        studyingmaze = True
        for a in range(self.numRows):
            for b in range(self.numCols):
                stats['dead_ends'] += self.maze[a][b].hasPaths(0)
                stats['two_open'] += self.maze[a][b].hasPaths(1)
                stats['three_open'] += self.maze[a][b].hasPaths(2)
                stats['four_open'] += self.maze[a][b].hasPaths(3)
                
        stats['total_squares'] = self.numRows * self.numCols

        studyingmaze = False
        return stats

    def _cull(self, paths, limit):
        print "Culling with " + str(paths) + " paths, limit " + str(limit)
        onePaths = self._list_with_paths(paths)
        random.shuffle(onePaths)

        totalOnePaths = len(onePaths)
        #print onePaths
        culled = 0

        currentCull = 0
        print totalOnePaths
        if totalOnePaths > 0:
            while (float(culled) / float(totalOnePaths) <= float(limit) ):
                didCull = 0
                print "Percentage culled: " + str(float( float(culled) / float(totalOnePaths) ))
                try:
                    x,y = onePaths[currentCull]
                except:
                    break
                availableForTeardown = self.maze[x][y].getBlocked()
    
                random.shuffle(availableForTeardown)
    
                while self.maze[x][y].hasPaths(paths):
                    # keep tearing until the cell has more than one path
                    # or we run out of directions available, such as on a corner
                    if len(availableForTeardown) > 0:
                        teardownWall = availableForTeardown.pop()
                        self.maze[x][y].ripWall(teardownWall)
                        didCull += 1
                    else:
                        break
                
                culled += didCull
                print culled
                currentCull += 1
        
    def _list_with_paths(self, paths):
        result = []
        for i in range(self.numRows):
            for j in range(self.numCols):
                if self.maze[i][j].hasPaths(paths):
                    result.append( (i,j) )
        return result

    def _count_unvisited(self):
        unvisited = 0
        
        for i in range(self.numRows):
            for j in range(self.numCols):
                if self.maze[i][j].visited is False:
                    unvisited += 1
        return unvisited
                        
    def _count_inaccessible(self):
        inaccessible = 0
        
        for i in range(self.numRows):
            for j in range(self.numCols):
                if self.maze[i][j].inaccessible() is True:
                    inaccessible += 1
        return inaccessible
        
    # Takes a cell and a direction.  The direction indicates the direction
    # we want to go
    def _make_path( self, cell):
        # Pick a random direction to go
        directions = cell.GetUnvisited()
        
        # bias
        for a in range(len(directions)):
            for b in range(a):
                directions.append(directions[a])
        # Shuffle directions
        random.shuffle(directions)
        #print directions

        if len(directions) > 0:
            # Now knock out the wall in that direction and go
            # We still need to check if the cell was visited because it may have
            # been visited since we built the list of unvisited directions
            for newDir in directions:
                newCell = cell.getCell(newDir)
                if newCell is not None and newCell.visited is not True:
                    cell.ripWall(newDir)
                    newCell.Visit()
                    self._make_path( newCell )

    # radius should be a ratio, numControl Points are the points wanted for
    # the circle.  A list of control points in maze coordinates will be returned
    def _get_circle(self, radius, numControlPoints, xAxis, yAxis, aspectRatio = 1.0):
        controlPoints = []
        xRadius = float(radius*xAxis/2.0)# * float(aspectRatio)
        yRadius = float(radius*yAxis/2.0)# / float(aspectRatio)
        for ang in range(0, numControlPoints):
            angle = float(ang)/float(numControlPoints) * 2.0*math.pi
            x,y = (math.cos(angle) * xRadius, math.sin(angle) * yRadius)
            controlPoints.append( (x,y) )

        return controlPoints
    
    def PlaceSpawnPoints(self):
        if self.numCols > self.numRows:
            directionList = [RIGHT, LEFT]
        else:
            directionList = [DOWN, UP]
        spawnPoints = self._get_circle(self.spawnRadius,
                                       self.spawnPoints,
                                       self.numCols, self.numRows,
                                       float(self.numRows)/float(self.numCols) )
        if self.game_mode == 0:
            for point in spawnPoints:
                x,y = int(point[0]), int(point[1])
                direction = int(x > y)
                x = x + self.numCols/2
                y = y + self.numRows/2
                self.maze[y][x].setSpawn(directionList[ direction ] )
        if self.game_mode == 1:
            self.maze[0][0].setSpawn(UP)
    
    def PlaceZones(self):
        if self.game_mode == 0:
            if self.zones is not None:
                if isinstance(self.zones, list):
                    # First determine if we have a position for the zones
                    # If any zone does not have a position, then all zones
                    # will be computed.
                    numPos = 0
                    for zone in self.zones:
                        if zone.has_key('loc'):
                            numPos += 1
                    if len(self.zones) != numPos:
                        zonePoints = self._get_circle(self.zoneDiameter/2.0, len(self.zones), self.numRows/self.numCols )
                        for zone in self.zones:
                            zone['pos'] = zonePoints.pop()
                    for zone in self.zones:
                        self._place_zone(zone)
                elif isinstance(self.zones, map):
                    if zones.has_key('loc') is not True:
                        zones['loc'] = (0.5, 0.5)
                    self._place_zone(self.zones)
        if self.game_mode == 1:
            aZone = { 'loc' : (1.0, 1.0),
                      'effect' : 'win',
                      'radius' : 10,
                      'growth' : 0 }
            self._place_zone(aZone)

    def _place_zone(self, zone):
        '''
            zone['loc'] = (x,y) (ratios, with 0.0 being left, 1.0 being right, etc, not arma coordinates)
            zone['effect'] = win, fortress, death
            zone['radius'] = radius in arma coordinate units
            zone['growth'] = growth
        '''
        xmodif,ymodif = zone['loc']
        x = int( (self.numRows-1) * xmodif)
        y = int( (self.numCols-1) * ymodif)
        self.maze[x][y].SetZone(zone['radius'],zone['effect'],zone['growth'])
    
    def getMap(self):
        result = '''<?xml version="1.0" encoding="UTF-8" standalone="no"?>
<!DOCTYPE Resource SYSTEM "map-0.2.8_beta3.dtd">
<Resource type="aamap" name="testmap" version="0.1" author="Anonymous" category="mazes/generated">
    <Map version="0.2.8">
        <World>
            <Field>
                <Axes number="4"/>
'''
        spawnPoints = []
        zones = []
        verticalWalls = {}
        horizontalWalls = {}
        for i in range(self.numRows):
            for j in range(self.numCols):
                # Get spawn point first, if it has one
                spawnPoint = self.maze[i][j].GetSpawn(i,j, self.xUnitSize, self.yUnitSize)
                zone = self.maze[i][j].GetZone(i,j, self.xUnitSize, self.yUnitSize)
                if spawnPoint is not None:
                    spawnPoints.append(spawnPoint)
                if zone is not None:
                    zones.append(zone)
                # Now get the walls
                tmpWalls = self.maze[i][j].GetWalls(i, self.numCols - 1 - j, self.xUnitSize, self.yUnitSize)
                for a in tmpWalls:
                    if a.isHorizontal():
                        # These dictionaries are keyed to the horizontal
                        # or vertical that's stored in it
                        newKey = str(a.y1)
                        if not horizontalWalls.has_key(newKey):
                            horizontalWalls[newKey] = []
                        if a not in horizontalWalls[newKey]:
                            horizontalWalls[newKey].append(a)
                    elif a.isVertical():
                        newKey = str(a.x1)
                        if not verticalWalls.has_key(newKey):
                            verticalWalls[newKey] = []
                        if a not in verticalWalls[newKey]:
                            verticalWalls[newKey].append(a)

        walls = []
        # Sort each parallel and combine walls
        for a in verticalWalls.itervalues():
            a.sort()
            aCount = len(a)
            currentWall = 0
            for b in range(aCount):
                if a[currentWall].y2 == a[b].y1:
                    a[currentWall].y2 = a[b].y2
                else:
                    if a[currentWall] not in walls:
                        walls.append(a[currentWall])
                    currentWall = b
            if a[currentWall] not in walls:
                walls.append(a[currentWall])
        for a in horizontalWalls.itervalues():
            a.sort()
            aCount = len(a)
            currentWall = 0
            for b in range(aCount):
                if a[currentWall].x2 == a[b].x1:
                    a[currentWall].x2 = a[b].x2
                else:
                    if a[currentWall] not in walls:
                        walls.append(a[currentWall])
                    currentWall = b
            if a[currentWall] not in walls:
                walls.append(a[currentWall])
            
        # spawnpoints first
        for a in spawnPoints:
            result += a.getAsXML()
        # zones second
        for a in zones:
            result += a.getAsXML()
        # walls third
        for a in walls:
            result += a.getAsXML()
            
        result += '''
            </Field>
        </World>
    </Map>
</Resource>
'''
    
        return result

if __name__ == "__main__":
    from optparse import OptionParser
    
    parser = OptionParser()
    
    parser.add_option("-r", "--rows", action="store", type="int", default=10, help="number of rows", dest="rows")
    parser.add_option("-c", "--cols", action="store", type="int", default=10, help="number of cols", dest="cols")
    parser.add_option("-s", "--spawns", action="store", type="int", default=4, help="number of spawn points", dest="spawns")
    parser.add_option("-C", "--cull", action="store", default="0.12,0.12,0.12", help="percentage of cells to cull, (deadends,one_paths,two_paths)", dest="pathcull")
    parser.add_option("-t", "--height", action="store", type="float", default=0.9, help="wall height", dest="wallheight")
    parser.add_option("-d", "--dead-end-cull", action="store", type="float", default=0.12, help="percentage of cells with only one path to convert to two/three paths", dest="deadendcull")
    parser.add_option("-f", "--format", action="store", default="ascii", help="format to generate (ascii, xml)", dest="format")
    parser.add_option("-o", "--output-file", action="store", default="NONE", help="file to save as (only works with xml)", dest="outputFile")
    parser.add_option("-m", "--mode", action="store", default="1", type="int", help="game mode to generate.  0: Solve the maze.  1: Race", dest="gameMode")
    (options, args) = parser.parse_args()
    
    numRows = options.rows
    pathCull = options.pathcull.split(',')
    for a in pathCull:
        a = float(a)
    deadEndCull = options.deadendcull
    numCols = options.cols
    format = options.format
    outputFile = options.outputFile
    wallHeight = options.wallheight
    spawnpoints = options.spawns
    gameMode = options.gameMode
    
    theMaze = Maze( rows=numRows,
                    cols=numCols,
                    two_path_cull=pathCull[2],
                    one_path_cull=pathCull[1],
                    dead_end_cull=pathCull[0],
                    wall_height=wallHeight,
                    spawn_points=spawnpoints,
                    game_mode = gameMode)
    try:
        theMaze.GenerateMaze()
        if format=="xml":
            if outputFile != "NONE":
                theFile = open(outputFile,'w')
                theFile.write(theMaze.getMap())
                theFile.close
            else:
                print theMaze.getMap()
            print "Dead ends: " + str(theMaze.rawStatistics['dead_ends']) + " : " + str(theMaze.finishedStatistics['dead_ends'])
            print "One path: " + str(theMaze.rawStatistics['two_open']) + " : " + str(theMaze.finishedStatistics['two_open'])
            print "Two paths: " + str(theMaze.rawStatistics['three_open']) + " : " + str(theMaze.finishedStatistics['three_open'])
            print "Three paths: " + str(theMaze.rawStatistics['four_open']) + " : " + str(theMaze.finishedStatistics['four_open'])
            print "Total squares: " + str(theMaze.rawStatistics['total_squares'])
        else:
            print "Format not recognized."
    except MemoryError:
        print "Your maze is too big, so I didn't make it."





        